/**
 * 系统信息相关接口
 */
import { callNative, callNativeSync, mapBiliTheme } from './util';
import { compareVersion } from "./util/commonFn";
import * as eventHandlers from './util/eventHandlers';
export default {
    /**
     * 获取系统信息
     *
     * @param {object} options
     * @param {function} options.success 接口调用成功的回调
     */
    getSystemInfo(options = {}) {
        const originalSuccess = options.success;
        options.success = function (systemInfo = {}) {
            const appVersion = (systemInfo.version || '');
            const platform = (systemInfo.system || '').toLowerCase();
            if (platform.indexOf('android') > -1 && typeof appVersion === 'string' && compareVersion(appVersion, '6.7.0') >= 0 && compareVersion(appVersion, '6.8.0') < 0) {
            // Android 客户端 在v6.7版本中，遗漏SDKVersion升级的事情，导致v6.7版本和v6.6版本共享了同一个SDKVersion ： 3.12.0 。 应当升级到 3.13.0
                systemInfo.SDKVersion = '3.13.0';
            }
            systemInfo.biliTheme = mapBiliTheme(systemInfo.biliTheme);

            typeof originalSuccess === 'function' && originalSuccess.call(this, systemInfo);
        };

        // getSystemInfo没有complete回调，覆盖原始方法，避免触发该回调
        options.complete = function () {};
        callNative('getSystemInfo', options);
    },
    /**
     * 获取系统信息
     */
    getSystemInfoSync() {
        const systemInfo = callNativeSync('getSystemInfoSync').data || {};
        const appVersion = (systemInfo.version || '');
        const platform = (systemInfo.platform || '').toLowerCase();
        if (platform.indexOf('android') > -1 && typeof appVersion === 'string' && compareVersion(appVersion, '6.7.0') >= 0 && compareVersion(appVersion, '6.8.0') < 0) {
            // Android 客户端 在v6.7版本中，遗漏SDKVersion升级的事情，导致v6.7版本和v6.6版本共享了同一个SDKVersion ： 3.12.0 。 应当升级到 3.13.0
            systemInfo.SDKVersion = '3.13.0';
        }

        systemInfo.biliTheme = mapBiliTheme(systemInfo.biliTheme);

        return systemInfo;
    },
    // 获取APP基础信息
    getAppBaseInfo () {
        const _sysInfo = bl.getSystemInfoSync() || {
            SDKVersion: '',
            enableDebug: '',
            host: '',
            language: '',
            version: '',
            theme: '',
        };
        let _output = {};

        if (compareVersion(_sysInfo.SDKVersion, '3.75.0') > -1) {
            const appTypeMap = {
                iphone: 0,
                android: 0,
                pangu_ios: 1,
                pangu_android: 1,
                ios_comic: 2,
                android_comic: 2
            };
            const _appBaseInfoForNA = callNativeSync('getAppBaseInfo') || {};
            const appBaseInfo = _appBaseInfoForNA.data ?  _appBaseInfoForNA.data : { host: { appKey: '' }};
            const { appKey } = appBaseInfo.host;
            appKey ? appBaseInfo.host.appType = appTypeMap[appKey] : appBaseInfo.host.appType = 0;
            delete appBaseInfo.host.appKey;
            _output = appBaseInfo;
        } else {
            const { SDKVersion, language, version } = _sysInfo;
            _output = {
                SDKVersion,
                enableDebug: false,
                host: {
                    appType: 0,
                    appName: '哔哩哔哩'
                },
                language,
                version,
            };
            // 兼容客户端逻辑，如果客户端返回了该字符，再返回给用户
            if (_sysInfo.theme) Object.assign(_output, {theme: _sysInfo.theme});
        }
        return _output;
    },

    onThemeChange(callback) {
        if(typeof callback !== 'function') {
            throw new Error('参数须是一个函数');
        }
        eventHandlers.systemHandlers.onThemeChange.push(callback);
    },

    offThemeChange(callback) {
        if(typeof callback !== 'function') {
            eventHandlers.systemHandlers.onThemeChange.splice(0, eventHandlers.systemHandlers.onThemeChange.length);
        } else {
            const index = eventHandlers.systemHandlers.onThemeChange.indexOf(callback);
            if(index >= 0) {
                eventHandlers.systemHandlers.onThemeChange.splice(index, 1);
            }
        }

    },

    showErrorPage(options = {}) {
        callNative('showErrorPage', options);
    },
};
